#include <stdio.h>
#include <math.h>
#include "aegulm.h"
#include "aeg_calc.h"

static int dimension = 0;     /* fuer FFT  */ 
static float *sinus = 0;
static float *cosinus = 0;

void hamwin(float in[], float out[], int length)
{   float   a  = 8*fatan(1)/length,
            c1 = 0.54,
            c2 = 0.46;
    static  bool    init = TRUE;    
    static  float   hamw[512];
    int   i,n;
    
    n = length/2;
    
    if(init)
    {   init = FALSE;
        if(length>512)
            ERROR("\n hamwin: %d > 512",length);
        for(i=0; i<n; i++)
            hamw[i] = c1 - c2 * fcos(a * (i+0.5) );
    }       
    for(i=0; i<n; i++)
    {   out[i]          = in[i]          * hamw[i]; 
        out[length-1-i] = in[length-1-i] * hamw[i];
    }
    return;
}

/***
****	FFT - Fast Fourier Transform
****
****	(nach einem Fortran-Programm in [Ahmed/Rao75], S.79)
****	komplexe Ein- & Ausgabe, FFT & IFFT,
****	f.d. Dimension 'n' sind NUR Zweierpotenzen zulaessig,
****	es wird FFT{x}(k) = 1/n * SUMME ... berechnet
****	(andere Autoren schreiben das '1/n' vor die IDFT)
****
****	Die Berechnung der SIN/COS erfolgt nur noch einmal.
****	Danach wird eine Tabelle abgerufen.
****					s.-talamazzini, 22/6/89
***/
/*****************************************************************/
void fft (                    	/* Fast Fourier Transform (FFT)  */
	  float xre[], float xim[],	/* Re/Im-Teile der Ein-/Ausgabe  */
	  int   n,              /* Dimension der FFT             */
	  int   invert)	        /* invert ? IFFT : FFT           */
/*****************************************************************/
{
int mit2, it2, iter, irem, it, nxp, nxp2, m, mxp, j1, j2, n2, i, j, k;
float wpwr, arg, wre, wim, tre, tim;
/***/
for (iter=0, irem=n/2 ; irem ; irem/=2, iter++)
	;
if (dimension != n)
	ini_fft (n);
for (it=0, nxp2=n, it2=1 ; it<iter ; it++, it2*=2) {
	nxp = nxp2;
	nxp2 = nxp/2;
	for (m=0, mit2=0 ; m<nxp2 ; m++, mit2+=it2) {
		wre = cosinus [ mit2 ];
		wim = invert  ?  sinus [mit2]  :  - sinus [mit2];
		for (mxp=nxp, j1=m ; mxp<=n ; mxp+=nxp, j1+=nxp) {
			j2 = j1 + nxp2;
			tre = xre[j1] - xre[j2];
			tim = xim[j1] - xim[j2];
			xre[j1] += xre[j2];
			xim[j1] += xim[j2];
			xre[j2] = tre*wre - tim*wim;
			xim[j2] = tre*wim + tim*wre;
			}
		}
	}
for (i=0, j=0 ; i<n-1 ; i++) {
	if (i < j) {
		tre = xre[j];
		xre[j] = xre[i];
		xre[i] = tre;
		tim = xim[j];
		xim[j] = xim[i];
		xim[i] = tim;
		}
	k = n/2;
	while (k <= j) {
		j -= k;
		k /= 2;
		}
	j += k;
	}
if (!invert)
	for (i=0 ; i<n ; i++) {
		xre[i] /= n;
		xim[i] /= n;
		}
}

/******************************************************************************/
static void ini_fft (int dim)
{   int n, dim2;
    float pidim2;

    dimension = dim;
    if (sinus)
	free (sinus);
    if (cosinus)
	free (cosinus);
    sinus   = (float*)calloc(sizeof(float), dim/2);
    cosinus = (float*)calloc(sizeof(float), dim/2);
    pidim2 = (2 * M_PI) / dim;
    for (n=0, dim2=dim/2 ; n<dim2 ; n++)
    {   cosinus [n] = fcos (pidim2 * n);
	sinus [n] = fsin (pidim2 * n);
    }
    return;
}

